-- version and date
local revision = tonumber(("$Revision: 63522 $"):match("%d+"))
if not NBB_REVISION or revision > NBB_REVISION then
    NBB_REVISION = revision
    NBB_DATE = ("$Date: 2008-03-04 02:26:20 -0500 (Tue, 04 Mar 2008) $"):match("%d%d%d%d%-%d%d%-%d%d")
end

-- localisation
local L = Rock("LibRockLocale-1.0"):GetTranslationNamespace("NBuffBars")
local LBS = LibStub("LibBabble-Spell-3.0")

-- libraries
local Tablet = AceLibrary("Tablet-2.0")

-- settings
NBuffBars:SetFuBarOption("iconPath", NBB_DEFAULT_ICON)
NBuffBars:SetFuBarOption("hasNoColor", true)
NBuffBars:SetFuBarOption("clickableTooltip", true)
NBuffBars:SetFuBarOption("hideWithoutStandby", true)
NBuffBars:SetFuBarOption("independentProfile", true)
NBuffBars:SetFuBarOption("tooltipType", "Tablet-2.0")

-- update text and icon
function NBuffBars:OnUpdateFuBarText()
    local currentBuff = NBB_BUFF[self.db.profile.cur]
    if currentBuff then
        self:SetFuBarText(currentBuff.name)
        self:SetFuBarIcon(currentBuff.icon)
    end
end

-- clicking on the fubar item
function NBuffBars:OnFuBarClick()

    self:ToggleVisibleKey()
end

function NBuffBars:AddToTooltip(name, list)

    local decoratedName = nil
    local currentBuff = nil

    local cat = Tablet:AddCategory(
        "id", name,
        "columns", 2,
        "text", name,
        "showWithoutChildren", false,
        "hideBlankLine", true,
        "textR", 1,
        "textG", 0.823529,
        "textB", 0,
        "child_hasCheck", true,
        "child_checked", true,
        "child_textR", 1,
        "child_textG", 1,
        "child_textB", 1,
        "child_indentation", 5
    )
    for i, v in pairs(list) do
        currentBuff = NBB_BUFF[v]
        if self.db.profile.cur == v then
            decoratedName = string.format("%s *", currentBuff.name)
        else
            decoratedName = currentBuff.name
        end
        cat:AddLine(
            "text", decoratedName,
            "func", self.SetBuff,
            "arg1", self,
            "arg2", v,
            "checkIcon", currentBuff.icon
        )
    end
end

-- tooltip formatting
function NBuffBars:OnUpdateFuBarTooltip()

    local profile = self.db.profile
    local hint = nil, nil

    if profile.visible and self:CanShow() then
        -- add a list of the interesting buffs
        self:AddToTooltip(L["Interesting Buffs"], profile.interesting)

        -- now the boring buffs
        self:AddToTooltip(L["Boring Buffs"], profile.boring)

        if self.exampleMember_m then
            hint = L["\n|cffeda55fLeft-Click|r to clear the example bars."]
        else
            hint = L["\n|cffeda55fLeft-Click|r to hide the bars."]
        end
    else
        hint = L["\n|cffeda55fLeft-Click|r to show the bars."]
    end
    
    hint = string.format("%s%s", hint, L["\n|cffeda55fRight-Click|r for options."])
    
    if self:IsFuBarMinimapAttached() then
        hint = string.format("%s%s", hint, L["\n|cffeda55fDrag|r the minimap icon to change its position."])
    end
    
    Tablet:SetHint(hint)
end
