-- version and date
local revision = tonumber(("$Revision: 63522 $"):match("%d+"))
if not NBB_REVISION or revision > NBB_REVISION then
    NBB_REVISION = revision
    NBB_DATE = ("$Date: 2008-03-04 02:26:20 -0500 (Tue, 04 Mar 2008) $"):match("%d%d%d%d%-%d%d%-%d%d")
end

-- localisation
local L = Rock("LibRockLocale-1.0"):GetTranslationNamespace("NBuffBars")

-- other libraries
local SharedMedia = LibStub("LibSharedMedia-2.0")

function NBuffBars:SetupOptions()
    self.options = {
    	name = self.name,
    	desc = self.notes,
    	icon = NBB_DEFAULT_ICON,
        type = "group",
        args = {
            groupBy = {
                name = L["Group By"],
                desc = L["Arrange the bars in groups."],
                type = "execute",
                func = NBuffBars.ToggleSort,
                passValue = NBuffBars,
                disabled = "~IsActive",
                order = 101,
            },
            makeInteresting = {
                name = L["Make Interesting"],
                desc = L["Change a Boring buff to Interesting."],
                type = "group",
                disabled = "~IsActive",
                order = 102,
                args = {},
            },
            makeBoring = {
                name = L["Make Boring"],
                desc = L["Change an Interesting buff to Boring."],
                type = "group",
                disabled = "~IsActive",
                order = 103,
                args = {},
            },
            spacer110 = {
                type = 'header',
                order = 110,
            },
            lock = {
                type = "toggle",
                name = L["Lock Position"],
                desc = L["Lock the position of the bars so that they cannot be dragged."],
                get = function()
                    return NBuffBars.db.profile.lock
                end,
                set = function(v)
                    NBuffBars.db.profile.lock = v
                    NBuffBars:UpdateLock()
                end,
                disabled = "~IsActive",
                order = 111,
            },
            resetPos = {
                type = "execute",
                name = L["Reset Position"],
                desc = L["Move the bars to the default position."],
                func = function()
                    NBuffBars:ResetAnchorPosition()
                end,
                disabled = "~IsActive",
                order = 112,
            },
            appearance = {
                name = L["Appearance"],
                desc = L["Change the appearance of the bars."],
                type = "group",
                disabled = "~IsActive",
                order = 113,
                args = {
                    textSize = {
                        name = L["Text Size"],
                        desc = L["Set the size of all text."],
                        type = "range",
                        get = function()
                            return NBuffBars.db.profile.appearance.textsize
                        end,
                        set = function(value)
                            NBuffBars.db.profile.appearance.textsize = value
                            NBuffBars:UpdateTextSizes()
                        end,
                        min = 8,
                        max = 24,
                        step = 1,
                        order = 101,
                    },
                    group = {
                        name = L["Show Group"],
                        desc = L["Show the group before the name."],
                        type = "toggle",
                        get = function()
                            return NBuffBars.db.profile.appearance.group
                        end,
                        set = function(value)
                            NBuffBars.db.profile.appearance.group = value
                            NBuffBars:UpdateBarPrefixes()
                        end,
                        order = 102,
                    },
                    texture = {
                        name = L["Texture"],
                        desc = L["Set the texture to use for the bars."],
                        type = "choice",
                        choices = SharedMedia:List('statusbar'),
                        choiceTextures = SharedMedia:HashTable('statusbar'),
                        get = function()
                            return NBuffBars.db.profile.appearance.texture
                        end,
                        set = function(value)
                            NBuffBars.db.profile.appearance.texture = value
                            NBuffBars:UpdateTextures()
                        end,
                        order = 103,
                    },
                    height = {
                        name = L["Bar Height"],
                        desc = L["Set the height of the bars."],
                        type = "range",
                        get = function()
                            return NBuffBars.db.profile.appearance.height
                        end,
                        set = function(value)
                            NBuffBars.db.profile.appearance.height = value
                            NBuffBars:UpdateGeo()
                        end,
                        min = 10,
                        max = 50,
                        step = 2,
                        order = 104,
                    },
                    width = {
                        name = L["Bar Width"],
                        desc = L["Set the width of the bars."],
                        type = "range",
                        get = function()
                            return NBuffBars.db.profile.appearance.width
                        end,
                        set = function(value)
                            NBuffBars.db.profile.appearance.width = value
                            NBuffBars:UpdateGeo()
                        end,
                        min = 100,
                        max = 800,
                        step = 10,
                        order = 105,
                    },
                    colour = {
                        name = L["Colour"],
                        desc = L["Change the colours."],
                        type = "group",
                        args = {
                            byClass = {
                                name = L["Names By Class"],
                                desc = L["Set the colour of the names according to their class."],
                                type = "toggle",
                                get = function()
                                    return NBuffBars.db.profile.appearance.colour.byClass
                                end,
                                set = function(value)
                                    NBuffBars.db.profile.appearance.colour.byClass = value
                                    NBuffBars:UpdateTextColours()
                                end,
                                order = 101,
                            },
                            text = {
                                name = L["Text"],
                                desc = L["Set the colour of the text."],
                                type = "color",
                                get = function()
                                    return unpack(NBuffBars.db.profile.appearance.colour.text)
                                end,
                                set = function(r, g, b, a)
                                    local t = NBuffBars.db.profile.appearance.colour.text
                                    t[1] = r
                                    t[2] = g
                                    t[3] = b
                                    t[4] = a
                                    NBuffBars:UpdateTextColours()
                                end,
                                hasAlpha = true,
                                order = 102,
                            },
                            header = {
                                name = L["Header"],
                                desc = L["Set the colour of the header."],
                                type = "color",
                                get = function()
                                    return unpack(NBuffBars.db.profile.appearance.colour.header)
                                end,
                                set = function(r, g, b, a)
                                    local t = NBuffBars.db.profile.appearance.colour.header
                                    t[1] = r
                                    t[2] = g
                                    t[3] = b
                                    t[4] = a
                                    if NBuffBars:IsCandyBarRegistered(NBB_HEADER_ID) then
                                        NBuffBars:SetCandyBarBackgroundColor(NBB_HEADER_ID, unpack(NBuffBars.db.profile.appearance.colour.header))
                                    end
                                end,
                                hasAlpha = true,
                                order = 103,
                            },
                            background = {
                                name = L["Background"],
                                desc = L["Set the colour of the background."],
                                type = "color",
                                get = function()
                                    return unpack(NBuffBars.db.profile.appearance.colour.frameBG)
                                end,
                                set = function(r, g, b, a)
                                    local t = NBuffBars.db.profile.appearance.colour.frameBG
                                    t[1] = r
                                    t[2] = g
                                    t[3] = b
                                    t[4] = a
                                    NBuffBars:UpdateBGColour()
                                end,
                                hasAlpha = true,
                                order = 104,
                            },
                            bar = {
                                name = L["Bar"],
                                desc = L["Set the colour of the bars."],
                                type = "color",
                                get = function()
                                    return unpack(NBuffBars.db.profile.appearance.colour.bar)
                                end,
                                set = function(r, g, b, a)
                                    local t = NBuffBars.db.profile.appearance.colour.bar
                                    t[1] = r
                                    t[2] = g
                                    t[3] = b
                                    t[4] = a
                                    NBuffBars:UpdateBarColours()
                                end,
                                hasAlpha = true,
                                order = 105,
                            },
                            barBackground = {
                                name = L["Bar Background"],
                                desc = L["Set the background colour of the bars."],
                                type = "color",
                                get = function()
                                    return unpack(NBuffBars.db.profile.appearance.colour.barBG)
                                end,
                                set = function(r, g, b, a)
                                    local t = NBuffBars.db.profile.appearance.colour.barBG
                                    t[1] = r
                                    t[2] = g
                                    t[3] = b
                                    t[4] = a
                                    NBuffBars:UpdateBarBGColours()
                                end,
                                hasAlpha = true,
                                order = 106,
                            },
    		                fade = {
    		                    name = L["Out Of Range Fade"],
    		                    desc = L["Set the factor by which the bars are faded when they are out of range."],
    		                    type = "range",
    		                    get = function()
    		                        return NBuffBars.db.profile.appearance.colour.fade
    		                    end,
    		                    set = function(value)
    		                        NBuffBars.db.profile.appearance.colour.fade = value
    								NBuffBars:UpdateBarColours()
    								NBuffBars:UpdateBarBGColours()
                                    NBuffBars:UpdateTextColours()
    		                    end,
    		                    min = 1,
    		                    max = 100,
    		                    step = 1,
    		                    order = 107,
    		                },
                        },
                        order = 106,
                    },
                    spacing = {
                        name = L["Spacing"],
                        desc = L["Change the spacing of the bars."],
                        type = "group",
                        args = {
                            bar = {
                                name = L["Bar Spacing"],
                                desc = L["Set the spacing between the bars."],
                                type = "range",
                                get = function()
                                    return NBuffBars.db.profile.appearance.spacing.bar
                                end,
                                set = function(value)
                                    NBuffBars.db.profile.appearance.spacing.bar = value
                                    NBuffBars:UpdateGeo()
                                end,
                                min = 0,
                                max = 20,
                                step = 1,
                                order = 101,
                            },
                            group = {
                                name = L["Group Spacing"],
                                desc = L["Set the spacing between the bar groups."],
                                type = "range",
                                get = function()
                                    return NBuffBars.db.profile.appearance.spacing.group
                                end,
                                set = function(value)
                                    NBuffBars.db.profile.appearance.spacing.group = value
                                    NBuffBars:UpdateGeo()
                                end,
                                min = 0,
                                max = 100,
                                step = 1,
                                order = 102,
                            },
                        },
                        order = 106,
                    },
                },
            },
            preset = {
                name = L["Load Preset"],
                desc = L["Load the Buff and Grouping options for a class or for raid leader."],
                type = "group",
                disabled = "~IsActive",
                order = 114,
                args = {},
            },
            showInParty = {
                name = L["Show In Party"],
                desc = L["Show the bars when in a party. Note that remaining times will only be accurate for buffs that you cast."],
                type = "toggle",
                disabled = "~IsActive",
                get = function()
                    return NBuffBars.db.profile.showInParty
                end,
                set = function(value)
                    NBuffBars.db.profile.showInParty = value
                    NBuffBars:UpdateVisibility()
                    NBuffBars:UpdateHeader()
                    NBuffBars:UpdateGeo()
                    NBuffBars:UpdateFuBarTooltip()
                end,
                order = 116,
            },
            showSolo = {
                name = L["Show When Solo"],
                desc = L["Show the bars when soloing. Note that remaining times will only be accurate for buffs that you cast."],
                type = "toggle",
                disabled = "~IsActive",
                get = function()
                    return NBuffBars.db.profile.showSolo
                end,
                set = function(value)
                    NBuffBars.db.profile.showSolo = value
                    NBuffBars:UpdateVisibility()
                    NBuffBars:UpdateHeader()
                    NBuffBars:UpdateGeo()
                    NBuffBars:UpdateFuBarTooltip()
                end,
                order = 117,
            },
            example = {
                type = "execute",
                name = L["Show Example"],
                desc = L["Show the example bars."],
                func = function()
                    if NBuffBars.exampleMember_m then
                        NBuffBars:HideExample()
                    else
                        NBuffBars:ShowExample()
                    end
                end,
                disabled = "~IsActive",
                order = 118,
            },
        },
    }
    
    self:SetConfigTable(self.options)
end