-- version and date
local revision = tonumber(("$Revision: 64559 $"):match("%d+"))
if not NBB_REVISION or revision > NBB_REVISION then
    NBB_REVISION = revision
    NBB_DATE = ("$Date: 2008-03-15 03:25:22 -0400 (Sat, 15 Mar 2008) $"):match("%d%d%d%d%-%d%d%-%d%d")
end

-- localisation
local L = Rock("LibRockLocale-1.0"):GetTranslationNamespace("NBuffBars")

-- constants
local ICON = "Interface\\Glues\\CharacterCreate\\UI-CharacterCreate-Classes"
local ICON_POS = {
    WARRIOR = {0, 0.25, 0, 0.25},
    MAGE = {0.25, 0.49609375, 0, 0.25},
    ROGUE = {0.49609375, 0.7421875, 0, 0.25},
    DRUID = {0.7421875, 0.98828125, 0, 0.25},
    HUNTER = {0, 0.25, 0.25, 0.5},
    SHAMAN = {0.25, 0.49609375, 0.25, 0.5},
    PRIEST = {0.49609375, 0.7421875, 0.25, 0.5},
    WARLOCK = {0.7421875, 0.98828125, 0.25, 0.5},
    PALADIN = {0, 0.25, 0.5, 0.75},
}
local CLASS_COLOUR = {
    WARRIOR = { RAID_CLASS_COLORS["WARRIOR"].r, RAID_CLASS_COLORS["WARRIOR"].g, RAID_CLASS_COLORS["WARRIOR"].b, },
    MAGE = { RAID_CLASS_COLORS["MAGE"].r, RAID_CLASS_COLORS["MAGE"].g, RAID_CLASS_COLORS["MAGE"].b, },
    ROGUE = { RAID_CLASS_COLORS["ROGUE"].r, RAID_CLASS_COLORS["ROGUE"].g, RAID_CLASS_COLORS["ROGUE"].b, },
    DRUID = { RAID_CLASS_COLORS["DRUID"].r, RAID_CLASS_COLORS["DRUID"].g, RAID_CLASS_COLORS["DRUID"].b, },
    HUNTER = { RAID_CLASS_COLORS["HUNTER"].r, RAID_CLASS_COLORS["HUNTER"].g, RAID_CLASS_COLORS["HUNTER"].b, },
    SHAMAN = { RAID_CLASS_COLORS["SHAMAN"].r, RAID_CLASS_COLORS["SHAMAN"].g, RAID_CLASS_COLORS["SHAMAN"].b, },
    PRIEST = { RAID_CLASS_COLORS["PRIEST"].r, RAID_CLASS_COLORS["PRIEST"].g, RAID_CLASS_COLORS["PRIEST"].b, },
    WARLOCK = { RAID_CLASS_COLORS["WARLOCK"].r, RAID_CLASS_COLORS["WARLOCK"].g, RAID_CLASS_COLORS["WARLOCK"].b, },
    PALADIN = { RAID_CLASS_COLORS["PALADIN"].r, RAID_CLASS_COLORS["PALADIN"].g, RAID_CLASS_COLORS["PALADIN"].b, },
    PET = { 1, 1, 0, },
}
local DEFAULT_ICON = "Interface\\Icons\\Ability_Hunter_BeastCall"
local CAST_MACRO = SLASH_CAST1.." [target=%s] %s"
local USE_MACRO = SLASH_USE1.." [target=%s] %s"
local UNKNOWN_TIME = "??"

-- libraries
local SharedMedia = LibStub("LibSharedMedia-2.0")
local CandyBar = AceLibrary("CandyBar-2.0")

-- unique candybar identifier
local id = 0
local function getId()
    id = id + 1
    if id > 5000 then
        id = 0
    end
    return string.format("bb%d", id)
end

-- frame pool
local framepool = {}

-- function to return time left: <1s: empty, <1m: seconds, otherwise: minutes
local function ReturnTimerText(elapsed, unknown)

    if unknown then
        return UNKNOWN_TIME
    elseif (elapsed > 60) then
        return string.format("%dm", floor((elapsed + 30) / 60))
    elseif (elapsed > 1) then
        return string.format("%ds", elapsed)
    else
        return nil
    end
end

-- create the class
RaidMember = {}

-- members
RaidMember.barId_m = nil
RaidMember.unitName_m = nil
RaidMember.unitId_m = nil
RaidMember.group_m = nil
RaidMember.unitClass_m = nil
RaidMember.selectFrame_m = nil
RaidMember.buff_m = nil
RaidMember.buffInfo_m = nil
RaidMember.remaining_m = nil
RaidMember.visible_m = false
RaidMember.icon_m = nil
RaidMember.inRange_m = nil
RaidMember.appearance_m = nil
RaidMember.isPlayer_m = false
RaidMember.selectExtendTop_m = 0

local debug = false

-- constructor
function RaidMember:new(appearance, unitName, unitId, unitClass, group, buff)
    if debug then nbbDebug("RaidMember:new unitName = "..tostring(unitName).." unitId = "..tostring(unitId).." unitClass = "..tostring(unitClass).." group = "..tostring(group).." buff = "..tostring(buff)) end

    -- OO init
    local o = {}
    setmetatable(o, self)
    self.__index = self
    self.__lt = RaidMember.LT
    
    -- libraries
    Rock("LibRockHook-1.0"):Embed(o)
    
    -- initialise members
    o.appearance_m = appearance
    o.unitName_m = unitName
    o.unitId_m = unitId
    o.unitClass_m = unitClass
    o.group_m = group
    o.buff_m = buff
    o.buffInfo_m = NBB_BUFF[buff]
    o.barId_m = getId()
    o:SetIconInfo()
    o.isPlayer_m = unitId and (UnitIsUnit(unitId, "player") == 1)
    
    if debug then nbbDebug("RaidMember: #framepool = "..tostring(#framepool)) end

    -- create the secure, invisible frame used to select the unit, grabbing from the frame pool if necessary
    if (#framepool > 0) then
        o.selectFrame_m = table.remove(framepool)
        if debug then nbbDebug("RaidMember: from pool selectFrame: "..tostring(o.selectFrame_m)) end
    else
        o.selectFrame_m = CreateFrame("Button", nil, UIParent, "SecureActionButtonTemplate")
        if debug then nbbDebug("RaidMember: created selectFrame: "..tostring(o.selectFrame_m)) end
    end
    o.selectFrame_m:SetFrameStrata("HIGH")
    o.selectFrame_m:Hide()

    -- hook for tooltips
    o:AddScriptHook(o.selectFrame_m, "OnEnter", "SelectFrameOnEnter")
    o:AddScriptHook(o.selectFrame_m, "OnLeave", "SelectFrameOnLeave")

    -- register left click for selection of the unit, but only if we have a valid unit i.e. not an example one
    if o.unitId_m then
        o.selectFrame_m:RegisterForClicks("LeftButtonUp", "RightButtonUp", "MiddleButtonUp")
        o.selectFrame_m:SetAttribute("unit", o.unitId_m)
        o.selectFrame_m:SetAttribute("type1", "target")
        o.selectFrame_m:SetAttribute("type2", "macro")
        o.selectFrame_m:SetAttribute("type3", "macro")
    end

    -- initialise the remaining times
    o.remaining_m = {}
    local remaining = nil
    for i, v in ipairs(NBB_BUFF_ORDER) do
        o.remaining_m[v] = {}
        remaining = o.remaining_m[v]
        remaining.left = nil
        remaining.start = nil
        remaining.isGroup = false
    end
    
    return o
end

-- destructor - must be explicitly called by the owner immediately before removing references to the object
function RaidMember:Delete()
    if debug then nbbDebug("RaidMember:Delete "..tostring(self.unitName_m)) end

    -- unregister the candybar, so it can be recycled by the library
    CandyBar:UnregisterCandyBar(self.barId_m)

    -- remove hooks
    self:RemoveHook(self.selectFrame_m, "OnEnter")
    self:RemoveHook(self.selectFrame_m, "OnLeave")

    -- hide the selection frame and put it in the frame pool
    self.selectFrame_m:Hide()
    table.insert(framepool, self.selectFrame_m)

    if debug then nbbDebug("RaidMember: #framepool = "..tostring(#framepool)) end

end

function RaidMember:SetIconInfo()
    self.icon_m = {}
    local iconPos = ICON_POS[self.unitClass_m]
    if iconPos then
        self.icon_m = { ICON, iconPos[1], iconPos[2], iconPos[3], iconPos[4], }
    else
        self.icon_m = { DEFAULT_ICON, }
    end
end

function RaidMember:SelectFrameOnEnter()
    if NBuffBars.dragging_m then return end
    local timeLeft, isGroup = self:GetTimeLeft()

    GameTooltip_SetDefaultAnchor(GameTooltip, UIParent)
    GameTooltip:ClearLines()
    
    if timeLeft == 1 then
        local spellText = nil
        if self.buffInfo_m.group then
            spellText = self.buffInfo_m.group
        else
            spellText = self.buffInfo_m.single
        end
        GameTooltip:AddLine(string.format(L["|cffffffff%s|r not present for |cff00ff00%s|r.\n"],
                            spellText, self.unitName_m))
    else
        local timeLeftText, spellText = nil, nil
        if timeLeft == 0 then
            timeLeftText = L["Unknown time"]
        else
            timeLeftText = string.format("%d:%02d", floor(timeLeft / 60), (timeLeft % 60)) 
        end
        if isGroup then
            spellText = self.buffInfo_m.group
        else
            spellText = self.buffInfo_m.single
        end
        GameTooltip:AddLine(string.format(L["|cffffffff%s|r remaining for |cffffffff%s|r for |cff00ff00%s|r.\n"],
                            timeLeftText, spellText, self.unitName_m))
    end

    if self.inRange_m then

        if self.buffInfo_m.castable then
            if self.buffInfo_m.item then
                GameTooltip:AddLine(string.format(L["|cffeda55fLeft-Click|r to select |cffffffff%s|r.\n|cffeda55fRight-Click|r to use |cffffffff%s|r."],
                                    self.unitName_m, self.buffInfo_m.item))
            else
                if self.buffInfo_m.group then
                    GameTooltip:AddLine(string.format(L["|cffeda55fLeft-Click|r to select |cffffffff%s|r.\n|cffeda55fRight-Click|r to cast |cffffffff%s|r.\n|cffeda55fMiddle-Click|r to cast |cffffffff%s|r."],
                                        self.unitName_m, self.buffInfo_m.single, self.buffInfo_m.group))
                else
                    GameTooltip:AddLine(string.format(L["|cffeda55fLeft-Click|r to select |cffffffff%s|r.\n|cffeda55fRight-Click|r to cast |cffffffff%s|r."],
                                        self.unitName_m, self.buffInfo_m.single))
                end
            end
        else
            GameTooltip:AddLine(string.format(L["|cffeda55fLeft-Click|r to select |cff00ff00%s|r."], self.unitName_m))
        end
    else

        GameTooltip:AddLine(string.format(L["|cff00ff00%s|r out of range."], self.unitName_m))
    end

    GameTooltip:Show()

    self.hooks[self.selectFrame_m].OnEnter()
end

function RaidMember:SelectFrameOnLeave()
    GameTooltip:Hide()

    self.hooks[self.selectFrame_m].OnLeave()
end

function RaidMember:Show()
    if InCombatLockdown() then return end

    if self.visible_m then return end
    if debug then nbbDebug("RaidMember:Show") end
    self.visible_m = true

    -- create the candy bar
    CandyBar:RegisterCandyBar(self.barId_m, 10, self:GetBarText(), ICON, 0, 0, 0)
    CandyBar:SetCandyBarFade(self.barId_m, -1)

    -- set the bar appearance
    self:UpdateTexture()
    self:UpdateFontSize()

    -- enable the unit select frame
    self.selectFrame_m:Show()

    -- scan our buffs
    self:ScanBuffs()
    
    -- do a range check
    self.inRange_m = nil
    self:CheckRange()
    
    -- set the click casts
    self:SetClickCasting()
    
end

function RaidMember:Hide()
    if InCombatLockdown() then return end

    if not self.visible_m then return end
    if debug then nbbDebug("RaidMember:Hide") end
    self.visible_m = false

    -- kill the candybar
    CandyBar:UnregisterCandyBar(self.barId_m)

    -- disable the unit select frame
    self.selectFrame_m:Hide()
end

-- set the height of the bar
function RaidMember:SetHeight(height)
    if InCombatLockdown() then return end

    CandyBar:SetCandyBarHeight(self.barId_m, height)
    self.selectFrame_m:SetHeight(height + self.selectExtendTop_m)

end

-- set the width of the bar
function RaidMember:SetWidth(width, iconWidth)
    if InCombatLockdown() then return end

    CandyBar:SetCandyBarWidth(self.barId_m, width)

    -- ensure the selection frame covers the icon
    self.selectFrame_m:SetWidth(width + iconWidth)

end

-- set the select area extension at the top
function RaidMember:SetSelectExtendTop(value)
    if InCombatLockdown() then return end
    
    self.selectExtendTop_m = value
end

-- set the anchor point
function RaidMember:SetPoint(point, frame, relativePoint, x, y)
    if InCombatLockdown() then return end

    CandyBar:SetCandyBarPoint(self.barId_m, point, frame, relativePoint, x, y)
    self.selectFrame_m:SetPoint(point, frame, relativePoint, x, y + self.selectExtendTop_m)
end

-- bar texture
function RaidMember:UpdateTexture()
    if debug then nbbDebug("RaidMember:UpdateTexture") end

    CandyBar:SetCandyBarTexture(self.barId_m, SharedMedia:Fetch(SharedMedia.MediaType.STATUSBAR, self.appearance_m.texture))

end

-- font size
function RaidMember:UpdateFontSize()
    if debug then nbbDebug("RaidMember:UpdateFontSize") end

    CandyBar:SetCandyBarFontSize(self.barId_m, self.appearance_m.textsize)

end

-- colour of the bar itself
function RaidMember:UpdateBarColour()
    if debug then nbbDebug("RaidMember:UpdateBarColour") end

    local colour = self.appearance_m.colour.bar

    -- set the colours
    if self.inRange_m then
        CandyBar:SetCandyBarColor(self.barId_m, unpack(colour, 1, 4))
    else
		CandyBar:SetCandyBarColor(self.barId_m, colour[1], colour[2], colour[3], colour[4] * self.appearance_m.colour.fade / 100)
    end
end

-- colour of the bar's background
function RaidMember:UpdateBarBGColour(colour)
    if debug then nbbDebug("RaidMember:UpdateBarBGColour") end

    local colour = self.appearance_m.colour.barBG

    -- set the colours
    if self.inRange_m then
        CandyBar:SetCandyBarBackgroundColor(self.barId_m, unpack(colour, 1, 4))
    else
		CandyBar:SetCandyBarBackgroundColor(self.barId_m, colour[1], colour[2], colour[3], colour[4] * self.appearance_m.colour.fade / 100)
    end
end

-- text colour
function RaidMember:UpdateTextColour()
    if debug then nbbDebug("RaidMember:UpdateTextColour") end

	local byClass = self.appearance_m.colour.byClass
    local colour = nil
	local defaultColour = self.appearance_m.colour.text
	
	if self.appearance_m.colour.byClass then
		-- set the class colour
		colour = CLASS_COLOUR[self.unitClass_m]
		if not colour then
			-- set the regular text colour if we can't find this class for whatever reason
			colour = defaultColour
		end
	else
		-- set the regular text colour
		colour = defaultColour
	end
	
	if self.inRange_m then
		CandyBar:SetCandyBarTextColor(self.barId_m, colour[1], colour[2], colour[3], defaultColour[4])
        CandyBar:SetCandyBarTimerTextColor(self.barId_m, colour[1], colour[2], colour[3], defaultColour[4])
	else
        local fade = self.appearance_m.colour.fade / 100
		CandyBar:SetCandyBarTextColor(self.barId_m, colour[1], colour[2], colour[3], defaultColour[4] * fade)
        CandyBar:SetCandyBarTimerTextColor(self.barId_m, colour[1], colour[2], colour[3], defaultColour[4] * fade)
	end
end

-- text prefix
function RaidMember:UpdatePrefix()
    if debug then nbbDebug("RaidMember:UpdatePrefix") end

    CandyBar:SetCandyBarText(self.barId_m, self:GetBarText())
end

-- set the time remaining on a buff
-- pass left==nil for no time remaining
function RaidMember:SetBuffTime(buff, isGroup, left)
    if debug then nbbDebug("RaidMember:SetBuffTime "..tostring(buff)..": "..tostring(left)) end

    -- store the time remaining on this buff
    self.remaining_m[buff].left = left
    if isGroup == true then
        self.remaining_m[buff].isGroup = true
    elseif isGroup == false then
        self.remaining_m[buff].isGroup = false
    end
    if left then
        self.remaining_m[buff].start = GetTime()
    else
        self.remaining_m[buff].start = nil
    end

    -- set the bar value
    self:UpdateBarValue()

end

-- change the buff we are displaying
function RaidMember:ChangeBuff(buff)
    if InCombatLockdown() then return end
    if not buff then return end

    self.buff_m = buff
    self.buffInfo_m = NBB_BUFF[buff]

    -- set the click casts
    self:SetClickCasting()
    
    -- set the bar value
    self:UpdateBarValue()

    -- set the colours
    self:CheckRange()

end

-- update the bar value
function RaidMember:UpdateBarValue()

    if debug then nbbDebug("RaidMember:UpdateBarValue "..tostring(self.unitName_m).." "..tostring(buff).." "..tostring(self:GetTimeLeft()).."/"..tostring(self.buffInfo_m.duration)) end

    -- update the candybar with the values for this buff
    CandyBar:StartCandyBar(self.barId_m)
    CandyBar:SetCandyBarTime(self.barId_m, self.buffInfo_m.duration)
    
    local timeLeft, isGroup = self:GetTimeLeft()
    if timeLeft == 0 then
        if debug then nbbDebug(" unknown time left") end
        if isGroup then
            CandyBar:SetCandyBarTimeLeft(self.barId_m, self.buffInfo_m.duration / 2)
        else
            CandyBar:SetCandyBarTimeLeft(self.barId_m, self.buffInfo_m.duration / 4)
        end
        CandyBar:SetCandyBarTimeFormat(self.barId_m, ReturnTimerText, true)
    else
        if debug then nbbDebug(" time left = "..tostring(timeLeft)) end
        CandyBar:SetCandyBarTimeLeft(self.barId_m, timeLeft)
        CandyBar:SetCandyBarTimeFormat(self.barId_m, ReturnTimerText, false)
    end

    -- need to reset the icon for some reason, after StartCandyBar is called
    CandyBar:SetCandyBarIcon(self.barId_m, unpack(self.icon_m))

end

-- set the click casting areas
function RaidMember:SetClickCasting()
    if InCombatLockdown() then return end
    if debug then nbbDebug("RaidMember:SetClickCasting") end
    
    if not self.unitId_m then return end

    if self.buffInfo_m.castable then
        if self.buffInfo_m.item then
            self.selectFrame_m:SetAttribute("macrotext2", USE_MACRO:format(self.unitId_m, self.buffInfo_m.item))
            self.selectFrame_m:SetAttribute("macrotext3", USE_MACRO:format(self.unitId_m, self.buffInfo_m.item))
        else
            local groupSpell = self.buffInfo_m.group and self.buffInfo_m.group or self.buffInfo_m.single
            self.selectFrame_m:SetAttribute("macrotext2", CAST_MACRO:format(self.unitId_m, self.buffInfo_m.single))
            self.selectFrame_m:SetAttribute("macrotext3", CAST_MACRO:format(self.unitId_m, groupSpell))
        end
    else
        self.selectFrame_m:SetAttribute("macrotext2", "")
        self.selectFrame_m:SetAttribute("macrotext3", "")
    end
end

-- check if we are in range to cast the current buff
function RaidMember:CheckRange()
    --if debug then nbbDebug("RaidMember:CheckRange") end

    local inRange = true

    -- default to "in range"
    if self.buffInfo_m.castable and self.unitId_m then
        if self.buffInfo_m.item then
            if GetItemCount(self.buffInfo_m.item) > 0 then
                -- unless the item to cast is not in range
                inRange = (IsItemInRange(self.buffInfo_m.item, self.unitId_m) == 1)
            end
        else
            -- or the spell is not in range
            if self.buffInfo_m.group then
                inRange = (IsSpellInRange(self.buffInfo_m.group, self.unitId_m) == 1)
            else
                inRange = (IsSpellInRange(self.buffInfo_m.single, self.unitId_m) == 1)
            end
        end
    end

    if self.inRange_m == nil or not (self.inRange_m == inRange) then
        if debug then nbbDebug(" applying in/out of range colours to "..self.unitName_m) end
        self.inRange_m = inRange
        self:UpdateBarColour()
        self:UpdateBarBGColour()
        self:UpdateTextColour()
    end
end

-- update the buffs and the time remaining, if possible
function RaidMember:ScanBuffs()
    if debug then nbbDebug("RaidMember:ScanBuffs "..tostring(self.unitName_m)) end

    if self.unitId_m then
        local remaining

        -- mark each of the buffs as unconfirmed
        for k, v in pairs(self.remaining_m) do
            v.seen = false
        end
        
        local i, j, buff, timeLeft = 1, nil, nil, nil
        while true do
            if self.isPlayer_m then
                j = GetPlayerBuff(i, "HELPFUL")
                buff = nil
                if j > 0 then
                    buff = GetPlayerBuffName(j)
                    timeLeft = GetPlayerBuffTimeLeft(j)
                end
            else
                buff, _, _, _, _, timeLeft = UnitBuff(self.unitId_m, i)
            end
            if not buff then break end
            
            -- update the time left if this is a buff we're interested in
            for k, v in pairs(NBB_BUFF) do
                if v.group == buff or v.single == buff then
                    remaining = self.remaining_m[k]
                    if remaining then
                        remaining.seen = true
                        
                        -- update the time left value if we have retrieved it
                        if timeLeft then
                            if debug then nbbDebug(" "..tostring(timeLeft).." left on "..tostring(k)) end
                            remaining.start = GetTime()
                            remaining.left = timeLeft
                            remaining.isGroup = (v.group == buff)
                        -- mark the buff as present, if it's not currently so, or it has apparently run out
                        elseif self:GetTimeLeft(k) <= 1 then
                            if debug then nbbDebug(" marking "..tostring(k).." as present, with unknown time left") end
                            remaining.start = GetTime()
                            remaining.left = nil
                            remaining.isGroup = (v.group == buff)
                        end
                    end
                    break
                end
            end
            
            i = i + 1
        end

        -- remove any unconfirmed buffs
        for k, v in pairs(self.remaining_m) do
            if not v.seen then
                v.left = nil
                v.start = nil
                v.isGroup = false
            end
        end
    end

    -- update the current bar
    self:UpdateBarValue()
end

-- set the group of the raid member
function RaidMember:SetGroup(group)
    self.group_m = group
    self:UpdatePrefix()
end

-- set the new unit ID
function RaidMember:SetUnitId(unitId)
    self.unitId_m = unitId

    -- update the select frame
    self.selectFrame_m:SetAttribute("unit", self.unitId_m)

    -- update the click casting
    self:SetClickCasting()

end

-- set the appearance table
function RaidMember:SetAppearance(appearance)
    self.appearance_m = appearance
end

-- return the time left on the current buff, and whether the group version of the buff is used
-- 0: time left unknown, but buff present
-- 1: buff not present
-- x: time left in seconds
function RaidMember:GetTimeLeft(buff)
    local remaining
    if buff then
        remaining = self.remaining_m[buff]
    else
        remaining = self.remaining_m[self.buff_m]
    end

    if remaining then
        if remaining.start then
            if remaining.left then
                local remainingTime = remaining.left - GetTime() + remaining.start
                if remainingTime < 0 then
                    remainingTime = 1
                end
                return remainingTime, remaining.isGroup
            else
                return 0, remaining.isGroup
            end
        else
            return 1, nil
        end
    else
        return 1, nil
    end
end

-- return the text that the bar should show
function RaidMember:GetBarText()
    if self.appearance_m.group then
        return string.format("%d: %s", self.group_m, self.unitName_m)
    else
        return self.unitName_m
    end
end

-- compare function, for sorting - by time first, then alphabetically
function RaidMember:LT(other)
    local timeLeft = self:GetTimeLeft()
    local otherTimeLeft = other:GetTimeLeft()

    -- value of 1 indicates not present, but 0 indicates unknown
    if timeLeft == 1 then
        timeLeft = -1
    end
    if otherTimeLeft == 1 then
        otherTimeLeft = -1
    end

    if timeLeft == otherTimeLeft then
        return self.unitName_m < other.unitName_m
    else
        return timeLeft < otherTimeLeft
    end
end
